package com.ngsoft.tt.dao.titan.access;

import com.thinkaurelius.titan.core.*;
import com.thinkaurelius.titan.core.attribute.Geoshape;
import com.thinkaurelius.titan.core.schema.Mapping;
import com.thinkaurelius.titan.core.schema.TitanGraphIndex;
import com.thinkaurelius.titan.core.schema.TitanManagement;
import commons.config.ConfigurationManager;
import org.apache.commons.configuration.Configuration;
import org.apache.http.util.TextUtils;
import org.apache.openejb.util.LogCategory;
import org.apache.openejb.util.Logger;

import java.time.Duration;
import java.util.stream.StreamSupport;

import static com.ngsoft.tt.commons.consts.TitanDaoConstants.*;

/**
 * Created by vitaly on 07/12/2016.
 */
public class TitanShcemaBeforeImport {

    private static final Logger LOGGER = Logger.getInstance(LogCategory.OPENEJB_STARTUP_CONFIG, TitanWrapper.class);
    private static final Configuration CONF = ConfigurationManager.loadLocalConfiguration("dao", "titan-db",
            commons.Constants.FileExt.properties);

    private static TitanGraph tg;

    public static  void main(String [] args) {
        LOGGER.info("Initializing titan...");

        try {
            LOGGER.info("Trying to connect titan instance...");
            tg = TitanFactory.open(CONF);
            LOGGER.info("Connected to titan instance.");
            if (tg.isOpen()) {
                LOGGER.info("Start Creating shcema...");
                createGraph();
                LOGGER.info("Stop Creating shcema...");
            }
        } catch (Throwable e) {
            LOGGER.error("Error while opening titan", e);
        }
    }


    private static void createGraph() {
        TitanManagement tm = tg.openManagement();


        /*----------------------------------------------- VERTEX + EDGES PROPERTIES ------------------------------------------*/
        //Example --> mgmt.makePropertyKey('nameXXX').dataType(String.class).make()
        PropertyKey pPrTokenId = createPropertyKey(tm, PR_TOKEN_ID, String.class, Cardinality.SINGLE);        //Create vertex properties and ttl label fot password reset token

        PropertyKey pNodeType = createPropertyKey(tm, NODE_TYPE, String.class, Cardinality.SINGLE);
        PropertyKey pCompanyNameSearch = createPropertyKey(tm, COMPANY_NAME_SEARCH, String.class, Cardinality.SINGLE);
        PropertyKey pCompanyName = createPropertyKey(tm, COMPANY_NAME, String.class, Cardinality.SINGLE);
        PropertyKey pStoryTitle = createPropertyKey(tm, STORY_TITLE, String.class, Cardinality.SINGLE);
        PropertyKey pStoryTitleSearch = createPropertyKey(tm, STORY_TITLE_SEARCH, String.class, Cardinality.SINGLE);
        PropertyKey pStoryContent = createPropertyKey(tm, STORY_CONTENT, String.class, Cardinality.SINGLE);
        PropertyKey pStoryContentSearch = createPropertyKey(tm, STORY_CONTENT_SEARCH, String.class, Cardinality.SINGLE);
        PropertyKey openPositionId = createPropertyKey(tm, OPEN_POSION_ID, String.class, Cardinality.SINGLE);
        PropertyKey openPositionTitleSearch = createPropertyKey(tm, OPEN_POSITION_TITLE_SEARCH, String.class, Cardinality.SINGLE);
        PropertyKey openPositionCompanyNameSearch = createPropertyKey(tm, OPEN_POSITION_COMPANY_NAME_SEARCH, String.class, Cardinality.SINGLE);
        PropertyKey pCategoryNameSearch = createPropertyKey(tm, CATEGORY_NAME_SEARCH, String.class, Cardinality.SINGLE);
        PropertyKey pEdgeauthorSearch = createPropertyKey(tm, EDGE_PROPERTY_AUTHOR_SEARCH, String.class, Cardinality.SINGLE);
        PropertyKey lastUpdate = createPropertyKey(tm, LAST_UPDATE, Long.class, Cardinality.SINGLE);
        PropertyKey pNotificationId = createPropertyKey(tm, NOTIFICATION_ID, Integer.class, Cardinality.SINGLE);
        PropertyKey pUserId = createPropertyKey(tm, USER_ID, String.class, Cardinality.SINGLE);
        PropertyKey creatorType = createPropertyKey(tm, STORY_CREATOR_TYPE, String.class, Cardinality.SINGLE);
        PropertyKey pUserEmail = createPropertyKey(tm, USER_EMAIL, String.class, Cardinality.SINGLE);
        PropertyKey pUserType = createPropertyKey(tm, USER_TYPE, String.class, Cardinality.SINGLE);
        PropertyKey userApplyPosition = createPropertyKey(tm, USER_APPLY_POSITION, String.class, Cardinality.SINGLE);
        PropertyKey pCompanyId = createPropertyKey(tm, COMPANY_ID, String.class, Cardinality.SINGLE);
        PropertyKey pIndustry = createPropertyKey(tm, COMPANY_INDUSTRY, String.class, Cardinality.SINGLE);
        PropertyKey pPromotion = createPropertyKey(tm, PROMOTION, Integer.class, Cardinality.SINGLE);
        PropertyKey pEmailSuffix = createPropertyKey(tm, COMPANY_EMAIL_SUFFIX, String.class, Cardinality.SINGLE);
        PropertyKey pStoryId = createPropertyKey(tm, STORY_ID, String.class, Cardinality.SINGLE);
        PropertyKey pstoryType = createPropertyKey(tm, STORY_TYPE, String.class, Cardinality.SINGLE);
        PropertyKey pCommentId = createPropertyKey(tm, COMMENT_ID, String.class, Cardinality.SINGLE);
        PropertyKey pCategoryName = createPropertyKey(tm, CATEGORY_NAME, String.class, Cardinality.SINGLE);
        PropertyKey pEducationId = createPropertyKey(tm, EDUCATION_ID, String.class, Cardinality.SINGLE);
        PropertyKey pEducationName = createPropertyKey(tm, EDUCATION_NAME, String.class, Cardinality.SINGLE);
        PropertyKey pEdgeType = createPropertyKey(tm, EDGE_TYPE, String.class, Cardinality.SINGLE);
        PropertyKey pPositionCompanyId = createPropertyKey(tm, POSITION_COMPANY_ID, String.class, Cardinality.SINGLE);
        PropertyKey pPositionCompanyName = createPropertyKey(tm, POSITION_COMPANY_NAME, String.class, Cardinality.SINGLE);
        PropertyKey openPositionVideos = createPropertyKey(tm, OPEN_POSITION_VIDEOS, String.class, Cardinality.SET);
        PropertyKey openPositionImages = createPropertyKey(tm, OPEN_POSITION_IMAGES, String.class, Cardinality.SET);
        PropertyKey pCompanyNameAttribute = createPropertyKey(tm, COMPANY_ATTRIBUTE_NAME, String.class, Cardinality.SINGLE);
        PropertyKey pEdgeNotificationId = createPropertyKey(tm, EDGE_PROPERTY_NOTIFICATION_ID, String.class, Cardinality.SINGLE);
        PropertyKey pEdgeWannaWorkId = createPropertyKey(tm, EDGE_PROPERTY_WANNA_WORK_ID, String.class, Cardinality.SINGLE);
        PropertyKey pEdgePositionId = createPropertyKey(tm, EDGE_PROPERTY_POSITION_ID, String.class, Cardinality.SINGLE);
        PropertyKey pEdgeWannaWorkType = createPropertyKey(tm, EDGE_TYPE, String.class, Cardinality.SINGLE);

        // Geoshape PropertyKey
        PropertyKey companyLocation = createPropertyKey(tm, COMPANY_LOCATION, Geoshape.class, Cardinality.SINGLE);
        PropertyKey openPositionLocation = createPropertyKey(tm, OPEN_POSITION_LOCATION_SEARCH, Geoshape.class, Cardinality.SINGLE);
        PropertyKey storyLocation = createPropertyKey(tm, STORY_LOCATION, Geoshape.class, Cardinality.SINGLE);
        PropertyKey userLocation = createPropertyKey(tm, USER_LOCATION, Geoshape.class, Cardinality.SINGLE);
        PropertyKey userTempFeedLocation = createPropertyKey(tm, USER_TEMP_FEED_LOCATION, Geoshape.class, Cardinality.SINGLE);


        createPropertyKey(tm, NOTIFICATION_TYPE_NAME, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, NOTIFICATION_FAMILY_ID, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, NAME, String.class, Cardinality.LIST);
        createPropertyKey(tm, USER_ROLES, String.class, Cardinality.SET);
        createPropertyKey(tm, USER_FIRST_NAME, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_LAST_NAME, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_PHONE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_PROFILE_IMAGE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_EMAIL_CONFIRMATION_STATUS, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_CONTACT_EMAIL, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_PASSWORD, String.class, Cardinality.SINGLE);//Should be unique email cross all users
        createPropertyKey(tm, USER_DATE_OF_BIRTH, Long.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_PREVIOUS_COMPANIES, String.class, Cardinality.SET); //Set of previous company names for single user
        createPropertyKey(tm, USER_COUNTRY, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_STATE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_CITY, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, ADDRESS, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_ZIP_CODE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_LANGUAGES, String.class, Cardinality.SET);//Set of languages for single user
        createPropertyKey(tm, WEB_PAGE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_FACEBOOK_ACCESS_TOKEN, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_LINKEDIN_ACCESS_TOKEN, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_LINKEDIN_ID, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_CV_LINK, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_PUSH_NOTIFICATION_TOKEN, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_REGISTRATION_DATE, Long.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_NOTIFICATION_SETTINGS, String.class, Cardinality.LIST);
        createPropertyKey(tm, USER_FIRST_LOGIN, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_FREE_TEXT, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, USER_SKILLS, String.class, Cardinality.LIST);
        createPropertyKey(tm, STATUS, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, ENABLED, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, DISABLED, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, IMAGES, String.class, Cardinality.LIST);
        createPropertyKey(tm, FULL_SCREEN_IMAGES, String.class, Cardinality.LIST);
        createPropertyKey(tm, SD_IMAGES, String.class, Cardinality.LIST);
        createPropertyKey(tm, HD_IMAGES, String.class, Cardinality.LIST);
        createPropertyKey(tm, EMAIL_IMAGE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, DESCRIPTION, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMPANY_FOUNDED, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMPANY_EMPLOYEES_NUM, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMPANY_FUNDING, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMPANY_VIBE_DISABLED, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMPANY_SNIPPETS, String.class, Cardinality.LIST);
        createPropertyKey(tm, COMPANY_BILLING_TYPE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMPANY_BILLING_LIKE_COUNT, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, STORY_VIDEO_LINK, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, STORY_VIDEO_DURATION, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, STORY_VIDEO_HLS_LINK, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, STORY_VIDEO_THUMBNAIL, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, STORY_EMAIL_VIDEO_THUMBNAIL, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, STORY_COLOR, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMMENT_CONTENT, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMMENT_IMAGE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, COMMENT_TIME, Long.class, Cardinality.SINGLE);
        createPropertyKey(tm, CATEGORY_SHOW_ALWAYS, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDUCATION_TYPE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, LOGO, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, DATA_SETS_X, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, LINES_PER_DATA_SET_X, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, DATA_SETS_Y, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, LINES_PER_DATA_SET_Y, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, GENERAL_QUESTION_LINE, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, NEWEST_GENERAL_QUESTION, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, QUESTION_FOR_COMPANY, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, TOP_POPULAR_USER_COMPANIES, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, TOP_SIMILAR_POPULAR_USER_COMPANIES, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, MAX_RANDOM_COMPANIES, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, MAX_RANDOM_STORIES, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, HARD_FACTS_STORY_LINE, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, CATEGORY_FEED_LINES, Integer.class, Cardinality.SINGLE);
        createPropertyKey(tm, WANNA_WORK_DAY_LIMIT, Integer.class, Cardinality.SINGLE);
        createVertexLabel(tm, VERTEX_LABEL_PR_TOKEN, Duration.ofHours(24));//NOTE 24 hours TTL
        createVertexLabel(tm, VERTEX_LABEL_TIMER, Duration.ofHours(24));//NOTE 24 hours TTL


        //Edge labels
        createEdgeLabel(tm, EDGE_LABEL_WORKED_AT, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_WANNA_WORK, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_LOOKING_FOR, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_STUDY, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_TEAM, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_LINKED_USER, Multiplicity.ONE2ONE);
        createEdgeLabel(tm, EDGE_LABEL_TAG, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_ABOUT_COMPANY, Multiplicity.MANY2ONE);
        createEdgeLabel(tm, EDGE_LABEL_LIKE, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_INTERESTED, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_FOLLOW, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_AUTHOR, Multiplicity.MANY2ONE);
        createEdgeLabel(tm, EDGE_LABEL_FOR, Multiplicity.MANY2ONE);
        createEdgeLabel(tm, EDGE_LABEL_VIEWED, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_OWNER, Multiplicity.MANY2ONE);
        createEdgeLabel(tm, EDGE_LABEL_COUNTER, Multiplicity.ONE2ONE);
        createEdgeLabel(tm, EDGE_LABEL_NOTIFIED, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_LINKED_COMPANY, Multiplicity.ONE2ONE);
        createEdgeLabel(tm, EDGE_LABEL_FAVORITE, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_OF_COMPANY, Multiplicity.MANY2ONE);
        createEdgeLabel(tm, EDGE_LABEL_CAN_PUBLISH_STORY, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_LOOKING_FOR, Multiplicity.MULTI);
        createEdgeLabel(tm, EDGE_LABEL_APPLY_POSION, Multiplicity.MULTI);

        //Edge properties
        createPropertyKey(tm, EDGE_PROPERTY_SINCE, Long.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_END, Long.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_STUDY_START, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_STUDY_END, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_DEGREE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_FIELD_OF_STUDY, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_OCCUPATION, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_JOB_DESCRIPTION, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_TM_TYPE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_TIME, Long.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_IS_INTERESTED, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_IS_FOLLOW, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_TIMES, Long.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_TYPE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_IS_CURRENT, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_TOKEN_TYPE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_IS_READ, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_IS_ANONYMOUS, Boolean.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_CONTENT, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_VERIFICATION_CODE, String.class, Cardinality.SINGLE);
        createPropertyKey(tm, EDGE_PROPERTY_VALIDATED, String.class, Cardinality.SINGLE);


        /**
         *  first boolean means --> isUnique
         *  second boolean means -> isCompositeIndex or not
         *  third param of "NULL" means not mixed index
         *  4th param is the refference to property key which is created before
         */
        createGraphIndex(tm, TitanVertex.class, true, true, null, pUserId);//User ID CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pUserEmail);//Email indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pCompanyId);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pEducationId);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pEducationName);//Education name indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pStoryId);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pCategoryName);//Category name indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pCommentId);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pPositionCompanyId);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pPositionCompanyName);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pCompanyNameAttribute);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pPrTokenId);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pPromotion);//Promotion indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pEdgeNotificationId);//Edge Notification indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pEdgePositionId);//Edge worked at (position) indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, true, true, null, pEmailSuffix);//CompositeIndex indexing and uniqueness
        createGraphIndex(tm, TitanVertex.class, false, true, null, userApplyPosition);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, openPositionImages);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, openPositionVideos);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, pNodeType);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, pUserType);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, pstoryType);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, creatorType);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, pIndustry);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanEdge.class, false, true, null, pEdgeWannaWorkId);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanVertex.class, false, true, null, pNotificationId);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanEdge.class, false, true, null, pEdgeWannaWorkType);//CompositeIndex indexing and NOT unique
        createGraphIndex(tm, TitanEdge.class, false, true, null, pEdgeType);//CompositeIndex indexing and NOT unique




          /*-----------------------------------------------  ELASTIC SEARCH INDECIES BASED ON VERTEX + EDGES PROPERTIES ------------------------------------------*/
        // company title
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, pCompanyNameSearch);//query search for backwords compatability // need to run after with hadoop reindex job
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, pCompanyName);//mixed index used for the company name text search
        // Story title  for search issues
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, pStoryTitle);//mixed index used for the story title text search
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, pStoryTitleSearch);//query search for backwords compatability //need to run after with hadoop reindex job
        // Story content  for search issues
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, pStoryContent);//mixed index used for the story title text search
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, pStoryContentSearch);//query search for backwords compatability // need to run after with hadoop reindex job

        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, openPositionTitleSearch);
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, openPositionCompanyNameSearch);

        // Category title for search issues
        createGraphIndex(tm, TitanVertex.class, false, false, false, "search", pNodeType, pCategoryNameSearch);//query search for backwords compatability // need to run after with hadoop reindex job
        createGraphIndex(tm, TitanEdge.class, false, false, false, "search", pNodeType, pEdgeauthorSearch);


         /*-----------------------------------------------  GEO LOCATION INDECIES ------------------------------------------*/
        // create geo company index
        String indexName1 = createIndexName(pNodeType, companyLocation);
        tm.buildIndex(indexName1, TitanVertex.class).addKey(pNodeType).addKey(companyLocation).buildMixedIndex("search");


        // create geo open position index
        String indexName2 = createIndexName(pNodeType, openPositionLocation);
        tm.buildIndex(indexName2, TitanVertex.class).addKey(pNodeType).addKey(openPositionLocation).buildMixedIndex("search");


        // create geo story index
        String indexName3 = createIndexName(pNodeType, storyLocation);
        tm.buildIndex(indexName3, TitanVertex.class).addKey(pNodeType).addKey(storyLocation).buildMixedIndex("search");


        // create user  index
        String indexName4 = createIndexName(pNodeType, userLocation);
        tm.buildIndex(indexName4, TitanVertex.class).addKey(pNodeType).addKey(userLocation).buildMixedIndex("search");


        // create user  index
        String indexName5 = createIndexName(pNodeType, userTempFeedLocation);
        tm.buildIndex(indexName5, TitanVertex.class).addKey(pNodeType).addKey(userTempFeedLocation).buildMixedIndex("search");


        tm.commit();

        tg.tx().rollback();

        LOGGER.info("FINISH BUILD INDEXES.....");
    }


    private static EdgeLabel createEdgeLabel(TitanManagement tm, String name, Multiplicity m) {
        EdgeLabel el = tm.getEdgeLabel(name);
        if (el != null) {
            LOGGER.info("Edge label {0} already exist.", name);
            return el;
        }
        return tm.makeEdgeLabel(name).multiplicity(m).make();
    }


    /**
     * Create vertex label with TTL
     *
     * @param tm       titan managment
     * @param name     vertex label name
     * @param timeUnit
     * @return
     */
    private static VertexLabel createVertexLabel(TitanManagement tm, String name, Duration timeUnit) {
        boolean exist = StreamSupport.stream(tm.getVertexLabels().spliterator(), false)
                .anyMatch(vl -> vl.name().equals(name));
        VertexLabel vertexLabel;
        if (exist) {
            LOGGER.info("Vertex label {0} already exist.", name);
            vertexLabel = tm.getVertexLabel(name);
        } else {
            LOGGER.info("New Vertex Label {0} created.", name);
            vertexLabel = tm.makeVertexLabel(name).setStatic().make();
        }
        if (vertexLabel.isStatic()) {
            tm.setTTL(vertexLabel, timeUnit);
            LOGGER.info("Vertex Label {0} TTL changed to {1}", name, timeUnit.toString());
        }
        return vertexLabel;
    }

    private static PropertyKey createPropertyKey(TitanManagement tm, String name, Class<?> dataTypeClass, Cardinality c) {
        //name = mgmt.makePropertyKey('name').dataType(String.class).make()
        PropertyKey key = tm.getPropertyKey(name);
        if (key != null) {
            LOGGER.info("Property key " + name + " already exist.");
            return key;
        }

        key = tm.makePropertyKey(name).dataType(dataTypeClass).cardinality(c).make();
        LOGGER.info("New Property key " + name + " created.");
        return key;
    }

    /**
     * @param tm
     * @param vertexOrEdge
     * @param isUnique
     * @param isCompositeIndex
     * @param mixedIndexType
     * @param keys
     */
    private static void createGraphIndex(TitanManagement tm, Class<? extends TitanElement> vertexOrEdge, boolean isUnique, boolean isCompositeIndex, String mixedIndexType, PropertyKey... keys) {
        createGraphIndex(tm, vertexOrEdge, isUnique, isCompositeIndex, false, mixedIndexType, keys);
    }

    /**
     * @param keys - create index name according keys names
     * @return
     */
    private static String createIndexName(PropertyKey... keys) {
        StringBuilder bob = new StringBuilder("index_");
        for (PropertyKey key : keys) {
            if (key != null && !TextUtils.isBlank(key.name())) {
                bob.append(key.name()).append('_');
            }
        }
        return bob.deleteCharAt(bob.length() - 1).toString();//delete last '_' char
    }

    /**
     * @param tm
     * @param vertexOrEdge
     * @param isUnique
     * @param isCompositeIndex
     * @param isReindex
     * @param mixedIndexType
     * @param keys
     */
    private static void createGraphIndex(TitanManagement tm, Class<? extends TitanElement> vertexOrEdge, boolean isUnique, boolean isCompositeIndex, boolean isReindex, String mixedIndexType, PropertyKey... keys) {
        String indexName = createIndexName(keys);
        if (validateIndex(tm, isReindex, indexName)) return;

        TitanManagement.IndexBuilder ib = tm.buildIndex(indexName, vertexOrEdge);
        if (mixedIndexType != null) {
            for (PropertyKey key : keys) {
                LOGGER.info("adding FULL TEXT key:{0} to index:{1}", key, indexName);
                ib.addKey(key, Mapping.TEXTSTRING.asParameter());
            }
        } else {
            //not mixedIndexType external index
            for (PropertyKey key : keys)
                ib.addKey(key);
        }

        if (isUnique)
            ib.unique();

        if (isCompositeIndex)
            ib.buildCompositeIndex();
        else {
            ib.buildMixedIndex(mixedIndexType);
        }
        //awaitGraphIndexStatus(tg, indexName);
        LOGGER.info("New Index {0} added to the graph.", indexName);
    }

    private static boolean validateIndex(TitanManagement tm, boolean isReindex, String indexName) {
        TitanGraphIndex index = tm.getGraphIndex(indexName);
        if (index != null) {
            LOGGER.info("Index {0} already exist", indexName);
            if (isReindex) {
                LOGGER.info("Trying to REINDEXING... {0}", indexName);
                // TODO Reindex
                // reindexKey(tm, keys);
                LOGGER.info("REINDEXING... {0} SUCCESS", indexName);
            }
            return true;
        }
        return false;
    }


}
